const db = require('../models/bootstrap')
var bcrypt = require("bcryptjs");
const Op = db.Sequelize.Op;
const User = db.user;
const Role = db.role;
const authConfig = require('../config/auth-config');
const emailConfig = require('../config/email-config');
const { registerEmail } = require('../services/emails/Email')

var jwt = require("jsonwebtoken");


// Get or search user by Keyword
getAllUsers = async (req, res) => {
    // const page_no = req.params.page_no;

    try {
        let search = '';
        const page_no = req.params.page_no ? req.params.page_no : 0;
        // console.log(req.query)

        if (req.query.search) {
            search = req.query.search
        }
        const total_row = 5;
        let page = 0;
        var wherecond = {}

        if (!Number.isNaN(page_no) && page_no > 0) {
            page = page_no
        }


        const user_roles = await User.findOne({
            where: { id: req._user_id },
            include: [{ model: Role, attributes: ['name', 'id'] }]
        })


        const role_name = user_roles.dataValues.roles[0].dataValues.name;


        if (search) {
            wherecond = {
                [Op.or]: [
                    {
                        email: {
                            [Op.like]: `%${search}%`
                        }
                    },

                    {
                        first_name: {
                            [Op.like]: `%${search}%`
                        }
                    }]
            }
        }

        if (role_name == 'Super Admin') {
            wherecond.id = { [Op.not]: req._user_id }
        } else if (role_name == 'admin') {
            wherecond.id = { [Op.and]: { [Op.not]: req._user_id, [Op.not]: 1 } }
        } else if (role_name == 'whitelabel') {
            wherecond.parent_user_id = { [Op.eq]: req._user_id }
        }
        // console.log(wherecond)




        const users = await User.findAndCountAll({
            limit: total_row,
            offset: page * total_row,

            order: [['createdAt', 'DESC']],
            where: wherecond
        });


        res.send({
            content: users.rows,
            totalPages: Math.ceil(users.count / total_row)
        })

    } catch (error) {
        console.log(error)
    }


}


// delete the user by ID
userDelete = async (req, res) => {
    try {
        const user = await User.findOne({ where: { id: req.body.user_id }, })

        // console.log(user)
        if (!user)
            res.status(404).send({ message: "User Not Found" })

        // console.log(req._ip)


        const response = await User
            .update(
                { active: 0, developer_ip: req._ip }
                , {
                    where: { id: req.body.user_id }
                }
            )

        // await User.destroy({
        //     where: { id: req.body.user_id }
        // });
        res.send({ message: "User deleted successfully." })

    } catch (error) {

    }
}

activateUser = async (req, res) => {
    try {
        const response = await User
            .update(
                { active: 1, developer_ip: req._ip }
                , {
                    where: { id: req.body.user_id }
                }
            )
        res.send({ message: "User Activated successfully." })
    } catch (error) {

    }
}


/* Create New user manually */

createUser = async (req, res) => {
    try {
        const { first_name, last_name, email, password, access } = req.body

        var role_id = 1;

        if (access.includes(',9')) {
            role_id = 2
        }



        const user = await User.findOne({
            where: {
                [Op.or]: [
                    {
                        email: {
                            [Op.like]: email
                        }
                    },

                    {
                        username: {
                            [Op.like]: email
                        }
                    }]
            }
        })


        if (user)
            return res.status(202).send({ status: 202, message: "User already exists" })



        const new_password = bcrypt.hashSync(password, Number(authConfig.bcryptSalt));

        const new_user = await User.create({
            username: email,
            email,
            password: new_password,
            active: 1,
            first_name,
            last_name,
            access: access,
            parent_user_id: req._user_id,
            developer_ip: req._ip
        })




        await new_user.setRoles([role_id]);

        registerEmail(email, password, emailConfig.email_register_subject, emailConfig.register_product_title)


        res.send({ message: "Created successfully" })



    } catch (error) {

    }
}



// Get Single User
getUser = async (req, res) => {
    try {
        const user_id = req.params.user_id;
        const user = await User.findOne({
            where: {
                id: user_id
            },
            attributes: ['id', 'first_name', 'email', 'last_name', 'access']
        })


        if (user)
            res.send(user)
    } catch (error) {

    }

}


// update the user by id

updateUser = async (req, res) => {
    try {
        const user_id = req.params.user_id;

        const { first_name, last_name, email, password, access } = req.body

        var role_id = 1;

        if (access.includes(',9')) {
            role_id = 2
        }

        const user = await User.findOne({
            where: { id: user_id },
        })

        var new_password = '';

        var updateData = {
            username: email,
            email,
            first_name,
            last_name,
            access: access,
            developer_ip: req._ip
        }
        if (password) {
            new_password = bcrypt.hashSync(password, Number(authConfig.bcryptSalt));
            updateData.password = new_password

        }



        // console.log(updateData)


        const response = await User
            .update(
                updateData,
                {
                    where: { id: user_id }
                }
            )



        await user.setRoles([role_id]);
        res.send({ message: "User Updated successfully" })
    } catch (error) {

    }
}

updateprofile = async (req, res) => {
    try {
        const { first_name, last_name, password, email } = req.body;
        const user = await User.findOne({
            where: { id: req._user_id },
        })


        if (!user)
            return res.status(404).send({ message: "User Not Found" })

        var updateData = {
            first_name,
            last_name
        }

        if (password.trim()) {
            updateData.password = bcrypt.hashSync(password, Number(authConfig.bcryptSalt))
        }


        const response = await User
            .update(
                updateData,
                {
                    where: { id: req._user_id }
                }
            )

        if (response) {
            var expiresTime = 86400; // 24 hours
            var token = jwt.sign({ id: user.id }, authConfig.secret, {
                expiresIn: expiresTime
            });
            roles = await user.getRoles();
            res.send({
                id: req._user_id,
                first_name,
                last_name,
                access: user.access,
                user_name: first_name + ' ' + last_name,
                token,
                expiresTime,
                email: email,
                roles: { role_id: roles[0].id, name: roles[0].name }
            })
        }

    } catch (error) {
        res.send({ message: error })
    }
}

module.exports = {
    getAllUsers,
    userDelete,
    activateUser,
    createUser,
    getUser,
    updateUser,
    updateprofile
}